Package: ChIPseeker
Type: Package
Title: ChIPseeker for ChIP peak Annotation, Comparison, and
        Visualization
Version: 1.45.0
Authors@R: c( 
    person(given = "Guangchuang", family = "Yu",       email = "guangchuangyu@gmail.com",      role  = c("aut", "cre"), comment = c(ORCID = "0000-0002-6485-8781")),
    person(given = "Ming",        family = "Li",       email = "limiang929@gmail.com",         role = "ctb"),
    person(given = "Qianwen",     family = "Wang",     email = "treywea@gmail.com",            role = "ctb"),
    person(given = "Yun",         family = "Yan",      email = "youryanyun@gmail.com",         role = "ctb"),
    person(given = "Hervé",       family = "Pagès",    email = "hpages.on.github@gmail.com",   role  = "ctb"),
    person(given = "Michael",     family = "Kluge",    email = "michael.kluge@bio.ifi.lmu.de", role  = "ctb"),
    person(given = "Thomas",      family = "Schwarzl", email = "schwarzl@embl.de",             role  = "ctb"),
    person(given = "Zhougeng",    family = "Xu",       email = "xuzhougeng@163.com",           role  = "ctb"),
	person(given = "Chun-Hui",    family = "Gao",      email="gaospecial@gmail.com",           role  = "ctb", comment=c(ORCID = "0000-0002-1445-7939"))
	)
Maintainer: Guangchuang Yu <guangchuangyu@gmail.com>
Description: This package implements functions to retrieve the nearest genes
    around the peak, annotate genomic region of the peak, statstical methods
    for estimate the significance of overlap among ChIP peak data sets, and
    incorporate GEO database for user to compare the own dataset with those
    deposited in database. The comparison can be used to infer cooperative
    regulation and thus can be used to generate hypotheses. Several
    visualization functions are implemented to summarize the coverage of the
    peak experiment, average profile and heatmap of peaks binding to TSS
    regions, genomic annotation, distance to TSS, and overlap of peaks or
    genes.
Depends: R (>= 4.1.0)
Imports: AnnotationDbi, aplot, BiocGenerics, boot, dplyr, enrichplot,
        IRanges, GenomeInfoDb, GenomicRanges, GenomicFeatures, ggplot2,
        gplots, graphics, grDevices, gtools, magrittr, methods,
        plotrix, parallel, RColorBrewer, rlang, rtracklayer, S4Vectors,
        scales, stats, tibble, TxDb.Hsapiens.UCSC.hg19.knownGene,
        utils, yulab.utils (>= 0.1.5)
Suggests: clusterProfiler, ggimage, ggplotify, ggupset, ggVennDiagram,
        knitr, org.Hs.eg.db, prettydoc, ReactomePA, rmarkdown, testthat
Remotes: GuangchuangYu/enrichplot
URL: https://yulab-smu.top/contribution-knowledge-mining/
BugReports: https://github.com/YuLab-SMU/ChIPseeker/issues
Encoding: UTF-8
VignetteBuilder: knitr
ByteCompile: true
License: Artistic-2.0
biocViews: Annotation, ChIPSeq, Software, Visualization,
        MultipleComparison
RoxygenNote: 7.3.2
git_url: https://git.bioconductor.org/packages/ChIPseeker
git_branch: devel
git_last_commit: 3fa2bfb
git_last_commit_date: 2025-04-15
Repository: Bioconductor 3.22
Date/Publication: 2025-10-07
NeedsCompilation: no
Packaged: 2025-10-07 20:56:29 UTC; biocbuild
Author: Guangchuang Yu [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-6485-8781>),
  Ming Li [ctb],
  Qianwen Wang [ctb],
  Yun Yan [ctb],
  Hervé Pagès [ctb],
  Michael Kluge [ctb],
  Thomas Schwarzl [ctb],
  Zhougeng Xu [ctb],
  Chun-Hui Gao [ctb] (ORCID: <https://orcid.org/0000-0002-1445-7939>)
