# Generated by roxygen2: do not edit by hand

S3method(arrange,GRanges)
S3method(as.data.frame,csAnno)
S3method(filter,GRanges)
S3method(mutate,GRanges)
S3method(rename,GRanges)
S3method(subset,csAnno)
export(.)
export(GRangesList)
export(annotatePeak)
export(as.GRanges)
export(combine_csAnno)
export(covplot)
export(downloadGEObedFiles)
export(downloadGSMbedFiles)
export(dropAnno)
export(enrichAnnoOverlap)
export(enrichPeakOverlap)
export(getAnnoStat)
export(getBioRegion)
export(getGEOInfo)
export(getGEOgenomeVersion)
export(getGEOspecies)
export(getPromoters)
export(getSampleFiles)
export(getTagMatrix)
export(makeBioRegionFromGranges)
export(overlap)
export(peakHeatmap)
export(peakHeatmap_multiple_Sets)
export(peak_Profile_Heatmap)
export(plotAnnoBar)
export(plotAnnoPie)
export(plotAnnoPie.csAnno)
export(plotAvgProf)
export(plotAvgProf2)
export(plotDistToTSS)
export(plotPeakProf)
export(plotPeakProf2)
export(readPeakFile)
export(rel)
export(seq2gene)
export(shuffle)
export(tagHeatmap)
export(vennpie)
export(vennplot)
export(vennplot.peakfile)
exportClasses(csAnno)
exportMethods(plotAnnoBar)
exportMethods(plotAnnoPie)
exportMethods(plotDistToTSS)
exportMethods(show)
exportMethods(upsetplot)
exportMethods(vennpie)
import(BiocGenerics)
import(GenomeInfoDb)
import(GenomicRanges)
import(IRanges)
import(S4Vectors)
importFrom(AnnotationDbi,get)
importFrom(AnnotationDbi,select)
importFrom(BiocGenerics,end)
importFrom(BiocGenerics,start)
importFrom(GenomicFeatures,exonsBy)
importFrom(GenomicFeatures,fiveUTRsByTranscript)
importFrom(GenomicFeatures,genes)
importFrom(GenomicFeatures,intronsByTranscript)
importFrom(GenomicFeatures,threeUTRsByTranscript)
importFrom(GenomicFeatures,transcripts)
importFrom(GenomicFeatures,transcriptsBy)
importFrom(GenomicRanges,GRangesList)
importFrom(S4Vectors,subset)
importFrom(TxDb.Hsapiens.UCSC.hg19.knownGene,TxDb.Hsapiens.UCSC.hg19.knownGene)
importFrom(aplot,insert_bottom)
importFrom(aplot,plot_list)
importFrom(boot,boot)
importFrom(boot,boot.ci)
importFrom(dplyr,arrange)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,mutate)
importFrom(dplyr,summarise)
importFrom(enrichplot,upsetplot)
importFrom(ggplot2,aes)
importFrom(ggplot2,aes_)
importFrom(ggplot2,aes_string)
importFrom(ggplot2,coord_fixed)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_blank)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_rect)
importFrom(ggplot2,geom_ribbon)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,labs)
importFrom(ggplot2,rel)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_brewer)
importFrom(ggplot2,scale_fill_distiller)
importFrom(ggplot2,scale_fill_hue)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_classic)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,xlab)
importFrom(ggplot2,xlim)
importFrom(ggplot2,ylab)
importFrom(gplots,plot.venn)
importFrom(grDevices,colorRampPalette)
importFrom(graphics,layout)
importFrom(graphics,legend)
importFrom(graphics,par)
importFrom(graphics,pie)
importFrom(graphics,plot.new)
importFrom(gtools,permutations)
importFrom(magrittr,"%<>%")
importFrom(magrittr,"%>%")
importFrom(methods,as)
importFrom(methods,is)
importFrom(methods,missingArg)
importFrom(methods,new)
importFrom(methods,show)
importFrom(parallel,detectCores)
importFrom(parallel,mclapply)
importFrom(plotrix,floating.pie)
importFrom(rlang,.data)
importFrom(rlang,quos)
importFrom(rtracklayer,import.chain)
importFrom(rtracklayer,liftOver)
importFrom(stats,p.adjust)
importFrom(stats,phyper)
importFrom(utils,data)
importFrom(utils,download.file)
importFrom(utils,read.delim)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
importFrom(yulab.utils,mat2df)
importFrom(yulab.utils,yulab_msg)
