#' @keywords internal
"_PACKAGE"



##' Information Datasets
##' 
##' ucsc genome version, precalcuated data and gsm information
##' 
##' @name info
##' @aliases ucsc_release
##' gsminfo
##' tagMatrixList
##' @docType data
##' @keywords datasets
NULL
