% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readPeakFile.R
\name{readPeakFile}
\alias{readPeakFile}
\title{readPeakFile}
\usage{
readPeakFile(peakfile, as = "GRanges", ...)
}
\arguments{
\item{peakfile}{peak file}

\item{as}{output format, one of GRanges or data.frame}

\item{...}{additional parameter (pass to `utils::read.delim()`)}
}
\value{
peak information, in GRanges or data.frame object
}
\description{
read peak file and store in data.frame or GRanges object
}
\examples{
peakfile <- system.file("extdata", "sample_peaks.txt", package="ChIPseeker")
peak.gr <- readPeakFile(peakfile, as="GRanges")
peak.gr
}
\author{
G Yu
}
