% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/csAnno.R
\name{getAnnoStat}
\alias{getAnnoStat}
\title{getAnnoStat}
\usage{
getAnnoStat(x)
}
\arguments{
\item{x}{csAnno object}
}
\description{
getting status of annotation
}
