% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/csAnno.R
\docType{class}
\name{csAnno-class}
\alias{csAnno-class}
\alias{show,csAnno-method}
\alias{vennpie,csAnno-method}
\alias{plotDistToTSS,csAnno-method}
\alias{plotAnnoBar,csAnno-method}
\alias{plotAnnoPie,csAnno-method}
\alias{upsetplot,csAnno-method}
\alias{subset,csAnno-method}
\title{Class "csAnno"
This class represents the output of ChIPseeker Annotation}
\description{
Class "csAnno"
This class represents the output of ChIPseeker Annotation
}
\section{Slots}{

\describe{
\item{\code{anno}}{annotation}

\item{\code{tssRegion}}{TSS region}

\item{\code{level}}{transcript or gene}

\item{\code{hasGenomicAnnotation}}{logical}

\item{\code{detailGenomicAnnotation}}{Genomic Annotation in detail}

\item{\code{annoStat}}{annotation statistics}

\item{\code{peakNum}}{number of peaks}
}}

\seealso{
\code{\link{annotatePeak}}
}
\author{
Guangchuang Yu \url{https://guangchuangyu.github.io}
}
\keyword{classes}
