% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/csAnno.R
\docType{methods}
\name{show}
\alias{show}
\alias{show,csAnno,ANY-method}
\title{show method}
\usage{
show(object)
}
\arguments{
\item{object}{A \code{csAnno} instance}
}
\value{
message
}
\description{
show method for \code{csAnno} instance
}
\author{
Guangchuang Yu \url{https://guangchuangyu.github.io}
}
