% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/csAnno.R
\docType{methods}
\name{vennpie}
\alias{vennpie}
\title{vennpie method generics}
\usage{
vennpie(x, r = 0.2, cex = 1.2, ...)

vennpie(x, r = 0.2, cex=1.2, ...)
}
\arguments{
\item{x}{A \code{csAnno} instance}

\item{r}{initial radius}

\item{cex}{value to adjust legend}

\item{...}{additional parameter}
}
\value{
plot
}
\description{
vennpie method generics
}
\author{
Guangchuang Yu \url{https://guangchuangyu.github.io}
}
