% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GEO.R
\name{getGEOInfo}
\alias{getGEOInfo}
\title{getGEOInfo}
\usage{
getGEOInfo(genome, simplify = TRUE)
}
\arguments{
\item{genome}{genome version}

\item{simplify}{simplify result or not}
}
\value{
data.frame
}
\description{
get subset of GEO information by genome version keyword
}
\author{
G Yu
}
