% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getGenomicAnnotation.R
\name{getGenomicAnnotation}
\alias{getGenomicAnnotation}
\title{getGenomicAnnotation}
\usage{
getGenomicAnnotation(
  peaks,
  distance,
  tssRegion = c(-3000, 3000),
  TxDb,
  level,
  genomicAnnotationPriority,
  sameStrand = FALSE
)
}
\arguments{
\item{peaks}{peaks in GRanges object}

\item{distance}{distance of peak to TSS}

\item{tssRegion}{tssRegion, default is -3kb to +3kb}

\item{TxDb}{TxDb object}

\item{level}{one of gene or transcript}

\item{genomicAnnotationPriority}{genomic Annotation Priority}

\item{sameStrand}{whether annotate gene in same strand}
}
\value{
character vector
}
\description{
get Genomic Annotation of peaks
}
\author{
G Yu
}
