% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/csAnno.R
\name{combine_csAnno}
\alias{combine_csAnno}
\title{combine_csAnno}
\usage{
combine_csAnno(x, ...)
}
\arguments{
\item{x}{csAnno object}

\item{...}{csAnno objects}
}
\value{
csAnno object
}
\description{
Combine csAnno Object
}
\details{
https://github.com/YuLab-SMU/ChIPseeker/issues/157
}
