% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{getSampleFiles}
\alias{getSampleFiles}
\title{getSampleFiles}
\usage{
getSampleFiles()
}
\value{
list of file names
}
\description{
get filenames of sample files
}
\author{
G Yu
}
