% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GEO.R
\name{getGEOspecies}
\alias{getGEOspecies}
\title{getGEOspecies}
\usage{
getGEOspecies()
}
\value{
data.frame
}
\description{
accessing species statistics collecting from GEO database
}
\author{
G Yu
}
